# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from instana_client.models.action_instance import ActionInstance
from instana_client.models.action_instance_request import ActionInstanceRequest
from instana_client.models.paginated_result import PaginatedResult

from instana_client.api_client import ApiClient, RequestSerialized
from instana_client.api_response import ApiResponse
from instana_client.rest import RESTResponseType


class ActionHistoryApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def add_action_instance(
        self,
        action_instance_request: ActionInstanceRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ActionInstance:
        """Run an automation action.

        Submits the automation action for execution on an agent. The automation action to execute and the agent on which to execute the action must be specified as actionId and hostId. For more details on the request payload see the request sample. This call requires `Execution of automation actions`  permission.

        :param action_instance_request: (required)
        :type action_instance_request: ActionInstanceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._add_action_instance_serialize(
            action_instance_request=action_instance_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ActionInstance",
            '400': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def add_action_instance_with_http_info(
        self,
        action_instance_request: ActionInstanceRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ActionInstance]:
        """Run an automation action.

        Submits the automation action for execution on an agent. The automation action to execute and the agent on which to execute the action must be specified as actionId and hostId. For more details on the request payload see the request sample. This call requires `Execution of automation actions`  permission.

        :param action_instance_request: (required)
        :type action_instance_request: ActionInstanceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._add_action_instance_serialize(
            action_instance_request=action_instance_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ActionInstance",
            '400': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def add_action_instance_without_preload_content(
        self,
        action_instance_request: ActionInstanceRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Run an automation action.

        Submits the automation action for execution on an agent. The automation action to execute and the agent on which to execute the action must be specified as actionId and hostId. For more details on the request payload see the request sample. This call requires `Execution of automation actions`  permission.

        :param action_instance_request: (required)
        :type action_instance_request: ActionInstanceRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._add_action_instance_serialize(
            action_instance_request=action_instance_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ActionInstance",
            '400': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _add_action_instance_serialize(
        self,
        action_instance_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if action_instance_request is not None:
            _body_params = action_instance_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/automation/actioninstances',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_action_instance(
        self,
        action_instance_id: Annotated[StrictStr, Field(description="Automation action run result ID to delete.")],
        var_from: Annotated[StrictInt, Field(description="From date filter in milliseconds (13-digit) to look up the action run result ID.")],
        to: Annotated[StrictInt, Field(description="To date filter in milliseconds (13-digit) to look up the action run result ID.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Deletes an automation action run result from the action run history by ID.

        Deletes the automation action run result specified in the request path from the action run history. The automation action run result will be deleted only if the action run result was created within the timeframe specified in the `from` and `to` query parameters. To get the automation action run result ID and the created time, you can run `/api/automation/actioninstances` and `/api/automation/actioninstances/<{actionInstanceId}>` REST calls. This call requires `Deletion of automation action history` permission.

        :param action_instance_id: Automation action run result ID to delete. (required)
        :type action_instance_id: str
        :param var_from: From date filter in milliseconds (13-digit) to look up the action run result ID. (required)
        :type var_from: int
        :param to: To date filter in milliseconds (13-digit) to look up the action run result ID. (required)
        :type to: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_action_instance_serialize(
            action_instance_id=action_instance_id,
            var_from=var_from,
            to=to,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '403': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_action_instance_with_http_info(
        self,
        action_instance_id: Annotated[StrictStr, Field(description="Automation action run result ID to delete.")],
        var_from: Annotated[StrictInt, Field(description="From date filter in milliseconds (13-digit) to look up the action run result ID.")],
        to: Annotated[StrictInt, Field(description="To date filter in milliseconds (13-digit) to look up the action run result ID.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Deletes an automation action run result from the action run history by ID.

        Deletes the automation action run result specified in the request path from the action run history. The automation action run result will be deleted only if the action run result was created within the timeframe specified in the `from` and `to` query parameters. To get the automation action run result ID and the created time, you can run `/api/automation/actioninstances` and `/api/automation/actioninstances/<{actionInstanceId}>` REST calls. This call requires `Deletion of automation action history` permission.

        :param action_instance_id: Automation action run result ID to delete. (required)
        :type action_instance_id: str
        :param var_from: From date filter in milliseconds (13-digit) to look up the action run result ID. (required)
        :type var_from: int
        :param to: To date filter in milliseconds (13-digit) to look up the action run result ID. (required)
        :type to: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_action_instance_serialize(
            action_instance_id=action_instance_id,
            var_from=var_from,
            to=to,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '403': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_action_instance_without_preload_content(
        self,
        action_instance_id: Annotated[StrictStr, Field(description="Automation action run result ID to delete.")],
        var_from: Annotated[StrictInt, Field(description="From date filter in milliseconds (13-digit) to look up the action run result ID.")],
        to: Annotated[StrictInt, Field(description="To date filter in milliseconds (13-digit) to look up the action run result ID.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Deletes an automation action run result from the action run history by ID.

        Deletes the automation action run result specified in the request path from the action run history. The automation action run result will be deleted only if the action run result was created within the timeframe specified in the `from` and `to` query parameters. To get the automation action run result ID and the created time, you can run `/api/automation/actioninstances` and `/api/automation/actioninstances/<{actionInstanceId}>` REST calls. This call requires `Deletion of automation action history` permission.

        :param action_instance_id: Automation action run result ID to delete. (required)
        :type action_instance_id: str
        :param var_from: From date filter in milliseconds (13-digit) to look up the action run result ID. (required)
        :type var_from: int
        :param to: To date filter in milliseconds (13-digit) to look up the action run result ID. (required)
        :type to: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_action_instance_serialize(
            action_instance_id=action_instance_id,
            var_from=var_from,
            to=to,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '403': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_action_instance_serialize(
        self,
        action_instance_id,
        var_from,
        to,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if action_instance_id is not None:
            _path_params['actionInstanceId'] = action_instance_id
        # process the query parameters
        if var_from is not None:
            
            _query_params.append(('from', var_from))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/automation/actioninstances/{actionInstanceId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_action_instances(
        self,
        var_from: Annotated[StrictInt, Field(description="From date filter in milliseconds (13-digit) to look up the action run results to delete")],
        to: Annotated[StrictInt, Field(description="To date filter in milliseconds (13-digit) to look up the action run results to delete")],
        target_snapshot_id: Annotated[Optional[StrictStr], Field(description="Target snapshot ID to look up the action run results to delete")] = None,
        event_id: Annotated[Optional[StrictStr], Field(description="Event ID filter to look up the action run results to delete")] = None,
        event_specification_id: Annotated[Optional[StrictStr], Field(description="Event specification ID filter to look up the action run results to delete")] = None,
        search: Annotated[Optional[StrictStr], Field(description="Text in action run result name, description and event name to filter the action run results to delete")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Action type filter to look up the action run results to delete")] = None,
        action_statuses: Annotated[Optional[List[StrictStr]], Field(description="Action status filter to look up the action run results to delete")] = None,
        action_ids: Annotated[Optional[List[StrictStr]], Field(description="List of action IDs  to filter the action run results to delete")] = None,
        policy_id: Annotated[Optional[StrictStr], Field(description="Policy ID filter to look up the action run results to delete")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete automation action run results.

        Deletes the automation action run results from the action run history. The results to delete is filtered by the filter values given in the query parameters. The automation action run result will be deleted only if the action run result was created within the timeframe specified in the `from` and `to` query parameters. This call requires `Deletion of automation action history` permission and, `from` and `to` query parameters.

        :param var_from: From date filter in milliseconds (13-digit) to look up the action run results to delete (required)
        :type var_from: int
        :param to: To date filter in milliseconds (13-digit) to look up the action run results to delete (required)
        :type to: int
        :param target_snapshot_id: Target snapshot ID to look up the action run results to delete
        :type target_snapshot_id: str
        :param event_id: Event ID filter to look up the action run results to delete
        :type event_id: str
        :param event_specification_id: Event specification ID filter to look up the action run results to delete
        :type event_specification_id: str
        :param search: Text in action run result name, description and event name to filter the action run results to delete
        :type search: str
        :param types: Action type filter to look up the action run results to delete
        :type types: List[str]
        :param action_statuses: Action status filter to look up the action run results to delete
        :type action_statuses: List[str]
        :param action_ids: List of action IDs  to filter the action run results to delete
        :type action_ids: List[str]
        :param policy_id: Policy ID filter to look up the action run results to delete
        :type policy_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_action_instances_serialize(
            var_from=var_from,
            to=to,
            target_snapshot_id=target_snapshot_id,
            event_id=event_id,
            event_specification_id=event_specification_id,
            search=search,
            types=types,
            action_statuses=action_statuses,
            action_ids=action_ids,
            policy_id=policy_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '403': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_action_instances_with_http_info(
        self,
        var_from: Annotated[StrictInt, Field(description="From date filter in milliseconds (13-digit) to look up the action run results to delete")],
        to: Annotated[StrictInt, Field(description="To date filter in milliseconds (13-digit) to look up the action run results to delete")],
        target_snapshot_id: Annotated[Optional[StrictStr], Field(description="Target snapshot ID to look up the action run results to delete")] = None,
        event_id: Annotated[Optional[StrictStr], Field(description="Event ID filter to look up the action run results to delete")] = None,
        event_specification_id: Annotated[Optional[StrictStr], Field(description="Event specification ID filter to look up the action run results to delete")] = None,
        search: Annotated[Optional[StrictStr], Field(description="Text in action run result name, description and event name to filter the action run results to delete")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Action type filter to look up the action run results to delete")] = None,
        action_statuses: Annotated[Optional[List[StrictStr]], Field(description="Action status filter to look up the action run results to delete")] = None,
        action_ids: Annotated[Optional[List[StrictStr]], Field(description="List of action IDs  to filter the action run results to delete")] = None,
        policy_id: Annotated[Optional[StrictStr], Field(description="Policy ID filter to look up the action run results to delete")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete automation action run results.

        Deletes the automation action run results from the action run history. The results to delete is filtered by the filter values given in the query parameters. The automation action run result will be deleted only if the action run result was created within the timeframe specified in the `from` and `to` query parameters. This call requires `Deletion of automation action history` permission and, `from` and `to` query parameters.

        :param var_from: From date filter in milliseconds (13-digit) to look up the action run results to delete (required)
        :type var_from: int
        :param to: To date filter in milliseconds (13-digit) to look up the action run results to delete (required)
        :type to: int
        :param target_snapshot_id: Target snapshot ID to look up the action run results to delete
        :type target_snapshot_id: str
        :param event_id: Event ID filter to look up the action run results to delete
        :type event_id: str
        :param event_specification_id: Event specification ID filter to look up the action run results to delete
        :type event_specification_id: str
        :param search: Text in action run result name, description and event name to filter the action run results to delete
        :type search: str
        :param types: Action type filter to look up the action run results to delete
        :type types: List[str]
        :param action_statuses: Action status filter to look up the action run results to delete
        :type action_statuses: List[str]
        :param action_ids: List of action IDs  to filter the action run results to delete
        :type action_ids: List[str]
        :param policy_id: Policy ID filter to look up the action run results to delete
        :type policy_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_action_instances_serialize(
            var_from=var_from,
            to=to,
            target_snapshot_id=target_snapshot_id,
            event_id=event_id,
            event_specification_id=event_specification_id,
            search=search,
            types=types,
            action_statuses=action_statuses,
            action_ids=action_ids,
            policy_id=policy_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '403': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_action_instances_without_preload_content(
        self,
        var_from: Annotated[StrictInt, Field(description="From date filter in milliseconds (13-digit) to look up the action run results to delete")],
        to: Annotated[StrictInt, Field(description="To date filter in milliseconds (13-digit) to look up the action run results to delete")],
        target_snapshot_id: Annotated[Optional[StrictStr], Field(description="Target snapshot ID to look up the action run results to delete")] = None,
        event_id: Annotated[Optional[StrictStr], Field(description="Event ID filter to look up the action run results to delete")] = None,
        event_specification_id: Annotated[Optional[StrictStr], Field(description="Event specification ID filter to look up the action run results to delete")] = None,
        search: Annotated[Optional[StrictStr], Field(description="Text in action run result name, description and event name to filter the action run results to delete")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Action type filter to look up the action run results to delete")] = None,
        action_statuses: Annotated[Optional[List[StrictStr]], Field(description="Action status filter to look up the action run results to delete")] = None,
        action_ids: Annotated[Optional[List[StrictStr]], Field(description="List of action IDs  to filter the action run results to delete")] = None,
        policy_id: Annotated[Optional[StrictStr], Field(description="Policy ID filter to look up the action run results to delete")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete automation action run results.

        Deletes the automation action run results from the action run history. The results to delete is filtered by the filter values given in the query parameters. The automation action run result will be deleted only if the action run result was created within the timeframe specified in the `from` and `to` query parameters. This call requires `Deletion of automation action history` permission and, `from` and `to` query parameters.

        :param var_from: From date filter in milliseconds (13-digit) to look up the action run results to delete (required)
        :type var_from: int
        :param to: To date filter in milliseconds (13-digit) to look up the action run results to delete (required)
        :type to: int
        :param target_snapshot_id: Target snapshot ID to look up the action run results to delete
        :type target_snapshot_id: str
        :param event_id: Event ID filter to look up the action run results to delete
        :type event_id: str
        :param event_specification_id: Event specification ID filter to look up the action run results to delete
        :type event_specification_id: str
        :param search: Text in action run result name, description and event name to filter the action run results to delete
        :type search: str
        :param types: Action type filter to look up the action run results to delete
        :type types: List[str]
        :param action_statuses: Action status filter to look up the action run results to delete
        :type action_statuses: List[str]
        :param action_ids: List of action IDs  to filter the action run results to delete
        :type action_ids: List[str]
        :param policy_id: Policy ID filter to look up the action run results to delete
        :type policy_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_action_instances_serialize(
            var_from=var_from,
            to=to,
            target_snapshot_id=target_snapshot_id,
            event_id=event_id,
            event_specification_id=event_specification_id,
            search=search,
            types=types,
            action_statuses=action_statuses,
            action_ids=action_ids,
            policy_id=policy_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '403': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_action_instances_serialize(
        self,
        var_from,
        to,
        target_snapshot_id,
        event_id,
        event_specification_id,
        search,
        types,
        action_statuses,
        action_ids,
        policy_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'types': 'multi',
            'actionStatuses': 'multi',
            'actionIds': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if var_from is not None:
            
            _query_params.append(('from', var_from))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        if target_snapshot_id is not None:
            
            _query_params.append(('targetSnapshotId', target_snapshot_id))
            
        if event_id is not None:
            
            _query_params.append(('eventId', event_id))
            
        if event_specification_id is not None:
            
            _query_params.append(('eventSpecificationId', event_specification_id))
            
        if search is not None:
            
            _query_params.append(('search', search))
            
        if types is not None:
            
            _query_params.append(('types', types))
            
        if action_statuses is not None:
            
            _query_params.append(('actionStatuses', action_statuses))
            
        if action_ids is not None:
            
            _query_params.append(('actionIds', action_ids))
            
        if policy_id is not None:
            
            _query_params.append(('policyId', policy_id))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/automation/actioninstances',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_action_instance(
        self,
        action_instance_id: Annotated[StrictStr, Field(description="action run result ID to get action run result details")],
        window_size: Annotated[Optional[StrictInt], Field(description="Window size in milliseconds. This value is used compute the from date `(from = to - windowSize)` to get the action run result details. The default `windowSize` is set to 10 minutes if this value is not provided.")] = None,
        to: Annotated[Optional[StrictInt], Field(description="To date filter in milliseconds (13-digit) to get the action run result details. The default `to` date is set to `System.currentTimeMillis()` if this value is not provided.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ActionInstance:
        """Get the details of an automation action run result by ID from action run history.

        Get the details of automation action run result specified in the request path from action run history. The automation action run result will be fetched only if the action run result was created within the timeframe specified by the `windowSize` and `to` query parameters. The `windowSize` parameter is used to compute the from date (`from = to - windowSize`) of the timeframe. The default value for `windowSize` is 10 minutes and the default value for `to` is current system time. To get the automation action run result ID and the created time, you can run `/api/automation/actioninstances` REST calls. When using personal access tokens, the user must have at least `Viewer` access for Automation.

        :param action_instance_id: action run result ID to get action run result details (required)
        :type action_instance_id: str
        :param window_size: Window size in milliseconds. This value is used compute the from date `(from = to - windowSize)` to get the action run result details. The default `windowSize` is set to 10 minutes if this value is not provided.
        :type window_size: int
        :param to: To date filter in milliseconds (13-digit) to get the action run result details. The default `to` date is set to `System.currentTimeMillis()` if this value is not provided.
        :type to: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_action_instance_serialize(
            action_instance_id=action_instance_id,
            window_size=window_size,
            to=to,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ActionInstance",
            '400': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_action_instance_with_http_info(
        self,
        action_instance_id: Annotated[StrictStr, Field(description="action run result ID to get action run result details")],
        window_size: Annotated[Optional[StrictInt], Field(description="Window size in milliseconds. This value is used compute the from date `(from = to - windowSize)` to get the action run result details. The default `windowSize` is set to 10 minutes if this value is not provided.")] = None,
        to: Annotated[Optional[StrictInt], Field(description="To date filter in milliseconds (13-digit) to get the action run result details. The default `to` date is set to `System.currentTimeMillis()` if this value is not provided.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ActionInstance]:
        """Get the details of an automation action run result by ID from action run history.

        Get the details of automation action run result specified in the request path from action run history. The automation action run result will be fetched only if the action run result was created within the timeframe specified by the `windowSize` and `to` query parameters. The `windowSize` parameter is used to compute the from date (`from = to - windowSize`) of the timeframe. The default value for `windowSize` is 10 minutes and the default value for `to` is current system time. To get the automation action run result ID and the created time, you can run `/api/automation/actioninstances` REST calls. When using personal access tokens, the user must have at least `Viewer` access for Automation.

        :param action_instance_id: action run result ID to get action run result details (required)
        :type action_instance_id: str
        :param window_size: Window size in milliseconds. This value is used compute the from date `(from = to - windowSize)` to get the action run result details. The default `windowSize` is set to 10 minutes if this value is not provided.
        :type window_size: int
        :param to: To date filter in milliseconds (13-digit) to get the action run result details. The default `to` date is set to `System.currentTimeMillis()` if this value is not provided.
        :type to: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_action_instance_serialize(
            action_instance_id=action_instance_id,
            window_size=window_size,
            to=to,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ActionInstance",
            '400': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_action_instance_without_preload_content(
        self,
        action_instance_id: Annotated[StrictStr, Field(description="action run result ID to get action run result details")],
        window_size: Annotated[Optional[StrictInt], Field(description="Window size in milliseconds. This value is used compute the from date `(from = to - windowSize)` to get the action run result details. The default `windowSize` is set to 10 minutes if this value is not provided.")] = None,
        to: Annotated[Optional[StrictInt], Field(description="To date filter in milliseconds (13-digit) to get the action run result details. The default `to` date is set to `System.currentTimeMillis()` if this value is not provided.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get the details of an automation action run result by ID from action run history.

        Get the details of automation action run result specified in the request path from action run history. The automation action run result will be fetched only if the action run result was created within the timeframe specified by the `windowSize` and `to` query parameters. The `windowSize` parameter is used to compute the from date (`from = to - windowSize`) of the timeframe. The default value for `windowSize` is 10 minutes and the default value for `to` is current system time. To get the automation action run result ID and the created time, you can run `/api/automation/actioninstances` REST calls. When using personal access tokens, the user must have at least `Viewer` access for Automation.

        :param action_instance_id: action run result ID to get action run result details (required)
        :type action_instance_id: str
        :param window_size: Window size in milliseconds. This value is used compute the from date `(from = to - windowSize)` to get the action run result details. The default `windowSize` is set to 10 minutes if this value is not provided.
        :type window_size: int
        :param to: To date filter in milliseconds (13-digit) to get the action run result details. The default `to` date is set to `System.currentTimeMillis()` if this value is not provided.
        :type to: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_action_instance_serialize(
            action_instance_id=action_instance_id,
            window_size=window_size,
            to=to,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ActionInstance",
            '400': None,
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_action_instance_serialize(
        self,
        action_instance_id,
        window_size,
        to,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if action_instance_id is not None:
            _path_params['actionInstanceId'] = action_instance_id
        # process the query parameters
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/automation/actioninstances/{actionInstanceId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_action_instances(
        self,
        window_size: Annotated[Optional[StrictInt], Field(description="Window size filter in milliseconds (to compute the from date) to get the action run result details")] = None,
        to: Annotated[Optional[StrictInt], Field(description="To date filter in milliseconds (13-digit) to get the action run result details")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page to fetch -- used for paging the action run result records")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of records to return in each page -- used for paging the action run result records")] = None,
        target_snapshot_id: Annotated[Optional[StrictStr], Field(description="Target snapshot ID filter to get the action run result details")] = None,
        event_id: Annotated[Optional[StrictStr], Field(description="Event ID filter to get the action run result details")] = None,
        event_specification_id: Annotated[Optional[StrictStr], Field(description="Event specification ID filter to get the action run result details")] = None,
        search: Annotated[Optional[StrictStr], Field(description="Text in action run result name, description and event name filter to get the action run result details")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Action type filter to get the action run result details")] = None,
        action_statuses: Annotated[Optional[List[StrictStr]], Field(description="Action status filter to get the action run result details")] = None,
        action_ids: Annotated[Optional[List[StrictStr]], Field(description="Action IDs filter to get the action run result details for only these actions")] = None,
        policy_id: Annotated[Optional[StrictStr], Field(description="Policy ID filter to get the action run result details")] = None,
        order_by: Annotated[Optional[StrictStr], Field(description="Action run result column to order the result set.")] = None,
        order_direction: Annotated[Optional[StrictStr], Field(description="Sort order direction.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PaginatedResult:
        """Get the details of automation action run results from action run history.

        Get the details of automation action run results from action run history. The results are filtered by the filters in the query parameters. The automation action run result will be fetched only if the action run results were created within the timeframe specified by the `windowSize` and `to` query parameters. The `windowSize` parameter is used to compute the from date (`from = to - windowSize`) of the timeframe. The default value for `windowSize` is 10 minutes and the default value for `to` is current system time. When using personal access tokens, the user must have at least `Viewer` access for Automation and the automation action run results returned are also filtered based on the access set in `Limited access` permission settings. This API returns paginated result as specified in query parameters `page` and `pageSize`. The response contains list of automation action run results whose contents corresponds to the `page` query parameter and whose size corresponds to the `pageSize` query parameter.

        :param window_size: Window size filter in milliseconds (to compute the from date) to get the action run result details
        :type window_size: int
        :param to: To date filter in milliseconds (13-digit) to get the action run result details
        :type to: int
        :param page: Page to fetch -- used for paging the action run result records
        :type page: int
        :param page_size: Number of records to return in each page -- used for paging the action run result records
        :type page_size: int
        :param target_snapshot_id: Target snapshot ID filter to get the action run result details
        :type target_snapshot_id: str
        :param event_id: Event ID filter to get the action run result details
        :type event_id: str
        :param event_specification_id: Event specification ID filter to get the action run result details
        :type event_specification_id: str
        :param search: Text in action run result name, description and event name filter to get the action run result details
        :type search: str
        :param types: Action type filter to get the action run result details
        :type types: List[str]
        :param action_statuses: Action status filter to get the action run result details
        :type action_statuses: List[str]
        :param action_ids: Action IDs filter to get the action run result details for only these actions
        :type action_ids: List[str]
        :param policy_id: Policy ID filter to get the action run result details
        :type policy_id: str
        :param order_by: Action run result column to order the result set.
        :type order_by: str
        :param order_direction: Sort order direction.
        :type order_direction: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_action_instances_serialize(
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            target_snapshot_id=target_snapshot_id,
            event_id=event_id,
            event_specification_id=event_specification_id,
            search=search,
            types=types,
            action_statuses=action_statuses,
            action_ids=action_ids,
            policy_id=policy_id,
            order_by=order_by,
            order_direction=order_direction,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PaginatedResult",
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_action_instances_with_http_info(
        self,
        window_size: Annotated[Optional[StrictInt], Field(description="Window size filter in milliseconds (to compute the from date) to get the action run result details")] = None,
        to: Annotated[Optional[StrictInt], Field(description="To date filter in milliseconds (13-digit) to get the action run result details")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page to fetch -- used for paging the action run result records")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of records to return in each page -- used for paging the action run result records")] = None,
        target_snapshot_id: Annotated[Optional[StrictStr], Field(description="Target snapshot ID filter to get the action run result details")] = None,
        event_id: Annotated[Optional[StrictStr], Field(description="Event ID filter to get the action run result details")] = None,
        event_specification_id: Annotated[Optional[StrictStr], Field(description="Event specification ID filter to get the action run result details")] = None,
        search: Annotated[Optional[StrictStr], Field(description="Text in action run result name, description and event name filter to get the action run result details")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Action type filter to get the action run result details")] = None,
        action_statuses: Annotated[Optional[List[StrictStr]], Field(description="Action status filter to get the action run result details")] = None,
        action_ids: Annotated[Optional[List[StrictStr]], Field(description="Action IDs filter to get the action run result details for only these actions")] = None,
        policy_id: Annotated[Optional[StrictStr], Field(description="Policy ID filter to get the action run result details")] = None,
        order_by: Annotated[Optional[StrictStr], Field(description="Action run result column to order the result set.")] = None,
        order_direction: Annotated[Optional[StrictStr], Field(description="Sort order direction.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PaginatedResult]:
        """Get the details of automation action run results from action run history.

        Get the details of automation action run results from action run history. The results are filtered by the filters in the query parameters. The automation action run result will be fetched only if the action run results were created within the timeframe specified by the `windowSize` and `to` query parameters. The `windowSize` parameter is used to compute the from date (`from = to - windowSize`) of the timeframe. The default value for `windowSize` is 10 minutes and the default value for `to` is current system time. When using personal access tokens, the user must have at least `Viewer` access for Automation and the automation action run results returned are also filtered based on the access set in `Limited access` permission settings. This API returns paginated result as specified in query parameters `page` and `pageSize`. The response contains list of automation action run results whose contents corresponds to the `page` query parameter and whose size corresponds to the `pageSize` query parameter.

        :param window_size: Window size filter in milliseconds (to compute the from date) to get the action run result details
        :type window_size: int
        :param to: To date filter in milliseconds (13-digit) to get the action run result details
        :type to: int
        :param page: Page to fetch -- used for paging the action run result records
        :type page: int
        :param page_size: Number of records to return in each page -- used for paging the action run result records
        :type page_size: int
        :param target_snapshot_id: Target snapshot ID filter to get the action run result details
        :type target_snapshot_id: str
        :param event_id: Event ID filter to get the action run result details
        :type event_id: str
        :param event_specification_id: Event specification ID filter to get the action run result details
        :type event_specification_id: str
        :param search: Text in action run result name, description and event name filter to get the action run result details
        :type search: str
        :param types: Action type filter to get the action run result details
        :type types: List[str]
        :param action_statuses: Action status filter to get the action run result details
        :type action_statuses: List[str]
        :param action_ids: Action IDs filter to get the action run result details for only these actions
        :type action_ids: List[str]
        :param policy_id: Policy ID filter to get the action run result details
        :type policy_id: str
        :param order_by: Action run result column to order the result set.
        :type order_by: str
        :param order_direction: Sort order direction.
        :type order_direction: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_action_instances_serialize(
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            target_snapshot_id=target_snapshot_id,
            event_id=event_id,
            event_specification_id=event_specification_id,
            search=search,
            types=types,
            action_statuses=action_statuses,
            action_ids=action_ids,
            policy_id=policy_id,
            order_by=order_by,
            order_direction=order_direction,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PaginatedResult",
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_action_instances_without_preload_content(
        self,
        window_size: Annotated[Optional[StrictInt], Field(description="Window size filter in milliseconds (to compute the from date) to get the action run result details")] = None,
        to: Annotated[Optional[StrictInt], Field(description="To date filter in milliseconds (13-digit) to get the action run result details")] = None,
        page: Annotated[Optional[StrictInt], Field(description="Page to fetch -- used for paging the action run result records")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of records to return in each page -- used for paging the action run result records")] = None,
        target_snapshot_id: Annotated[Optional[StrictStr], Field(description="Target snapshot ID filter to get the action run result details")] = None,
        event_id: Annotated[Optional[StrictStr], Field(description="Event ID filter to get the action run result details")] = None,
        event_specification_id: Annotated[Optional[StrictStr], Field(description="Event specification ID filter to get the action run result details")] = None,
        search: Annotated[Optional[StrictStr], Field(description="Text in action run result name, description and event name filter to get the action run result details")] = None,
        types: Annotated[Optional[List[StrictStr]], Field(description="Action type filter to get the action run result details")] = None,
        action_statuses: Annotated[Optional[List[StrictStr]], Field(description="Action status filter to get the action run result details")] = None,
        action_ids: Annotated[Optional[List[StrictStr]], Field(description="Action IDs filter to get the action run result details for only these actions")] = None,
        policy_id: Annotated[Optional[StrictStr], Field(description="Policy ID filter to get the action run result details")] = None,
        order_by: Annotated[Optional[StrictStr], Field(description="Action run result column to order the result set.")] = None,
        order_direction: Annotated[Optional[StrictStr], Field(description="Sort order direction.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get the details of automation action run results from action run history.

        Get the details of automation action run results from action run history. The results are filtered by the filters in the query parameters. The automation action run result will be fetched only if the action run results were created within the timeframe specified by the `windowSize` and `to` query parameters. The `windowSize` parameter is used to compute the from date (`from = to - windowSize`) of the timeframe. The default value for `windowSize` is 10 minutes and the default value for `to` is current system time. When using personal access tokens, the user must have at least `Viewer` access for Automation and the automation action run results returned are also filtered based on the access set in `Limited access` permission settings. This API returns paginated result as specified in query parameters `page` and `pageSize`. The response contains list of automation action run results whose contents corresponds to the `page` query parameter and whose size corresponds to the `pageSize` query parameter.

        :param window_size: Window size filter in milliseconds (to compute the from date) to get the action run result details
        :type window_size: int
        :param to: To date filter in milliseconds (13-digit) to get the action run result details
        :type to: int
        :param page: Page to fetch -- used for paging the action run result records
        :type page: int
        :param page_size: Number of records to return in each page -- used for paging the action run result records
        :type page_size: int
        :param target_snapshot_id: Target snapshot ID filter to get the action run result details
        :type target_snapshot_id: str
        :param event_id: Event ID filter to get the action run result details
        :type event_id: str
        :param event_specification_id: Event specification ID filter to get the action run result details
        :type event_specification_id: str
        :param search: Text in action run result name, description and event name filter to get the action run result details
        :type search: str
        :param types: Action type filter to get the action run result details
        :type types: List[str]
        :param action_statuses: Action status filter to get the action run result details
        :type action_statuses: List[str]
        :param action_ids: Action IDs filter to get the action run result details for only these actions
        :type action_ids: List[str]
        :param policy_id: Policy ID filter to get the action run result details
        :type policy_id: str
        :param order_by: Action run result column to order the result set.
        :type order_by: str
        :param order_direction: Sort order direction.
        :type order_direction: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_action_instances_serialize(
            window_size=window_size,
            to=to,
            page=page,
            page_size=page_size,
            target_snapshot_id=target_snapshot_id,
            event_id=event_id,
            event_specification_id=event_specification_id,
            search=search,
            types=types,
            action_statuses=action_statuses,
            action_ids=action_ids,
            policy_id=policy_id,
            order_by=order_by,
            order_direction=order_direction,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PaginatedResult",
            '403': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_action_instances_serialize(
        self,
        window_size,
        to,
        page,
        page_size,
        target_snapshot_id,
        event_id,
        event_specification_id,
        search,
        types,
        action_statuses,
        action_ids,
        policy_id,
        order_by,
        order_direction,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'types': 'multi',
            'actionStatuses': 'multi',
            'actionIds': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if window_size is not None:
            
            _query_params.append(('windowSize', window_size))
            
        if to is not None:
            
            _query_params.append(('to', to))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('pageSize', page_size))
            
        if target_snapshot_id is not None:
            
            _query_params.append(('targetSnapshotId', target_snapshot_id))
            
        if event_id is not None:
            
            _query_params.append(('eventId', event_id))
            
        if event_specification_id is not None:
            
            _query_params.append(('eventSpecificationId', event_specification_id))
            
        if search is not None:
            
            _query_params.append(('search', search))
            
        if types is not None:
            
            _query_params.append(('types', types))
            
        if action_statuses is not None:
            
            _query_params.append(('actionStatuses', action_statuses))
            
        if action_ids is not None:
            
            _query_params.append(('actionIds', action_ids))
            
        if policy_id is not None:
            
            _query_params.append(('policyId', policy_id))
            
        if order_by is not None:
            
            _query_params.append(('orderBy', order_by))
            
        if order_direction is not None:
            
            _query_params.append(('orderDirection', order_direction))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'ApiKeyAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/automation/actioninstances',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# flake8: noqa

# import apis into api package
from instana_client.api.ai_management_api import AIManagementApi
from instana_client.api.api_token_api import APITokenApi
from instana_client.api.action_catalog_api import ActionCatalogApi
from instana_client.api.action_history_api import ActionHistoryApi
from instana_client.api.apdex_report_api import ApdexReportApi
from instana_client.api.apdex_settings_api import ApdexSettingsApi
from instana_client.api.application_alert_configuration_api import ApplicationAlertConfigurationApi
from instana_client.api.application_analyze_api import ApplicationAnalyzeApi
from instana_client.api.application_catalog_api import ApplicationCatalogApi
from instana_client.api.application_metrics_api import ApplicationMetricsApi
from instana_client.api.application_resources_api import ApplicationResourcesApi
from instana_client.api.application_settings_api import ApplicationSettingsApi
from instana_client.api.application_topology_api import ApplicationTopologyApi
from instana_client.api.audit_log_api import AuditLogApi
from instana_client.api.authentication_api import AuthenticationApi
from instana_client.api.business_monitoring_api import BusinessMonitoringApi
from instana_client.api.custom_dashboards_api import CustomDashboardsApi
from instana_client.api.custom_entities_api import CustomEntitiesApi
from instana_client.api.end_user_monitoring_api import EndUserMonitoringApi
from instana_client.api.event_settings_api import EventSettingsApi
from instana_client.api.events_api import EventsApi
from instana_client.api.global_application_alert_configuration_api import GlobalApplicationAlertConfigurationApi
from instana_client.api.groups_api import GroupsApi
from instana_client.api.health_api import HealthApi
from instana_client.api.host_agent_api import HostAgentApi
from instana_client.api.infrastructure_alert_configuration_api import InfrastructureAlertConfigurationApi
from instana_client.api.infrastructure_analyze_api import InfrastructureAnalyzeApi
from instana_client.api.infrastructure_catalog_api import InfrastructureCatalogApi
from instana_client.api.infrastructure_metrics_api import InfrastructureMetricsApi
from instana_client.api.infrastructure_resources_api import InfrastructureResourcesApi
from instana_client.api.infrastructure_topology_api import InfrastructureTopologyApi
from instana_client.api.log_alert_configuration_api import LogAlertConfigurationApi
from instana_client.api.logging_analyze_api import LoggingAnalyzeApi
from instana_client.api.maintenance_configuration_api import MaintenanceConfigurationApi
from instana_client.api.mobile_app_analyze_api import MobileAppAnalyzeApi
from instana_client.api.mobile_app_catalog_api import MobileAppCatalogApi
from instana_client.api.mobile_app_configuration_api import MobileAppConfigurationApi
from instana_client.api.mobile_app_metrics_api import MobileAppMetricsApi
from instana_client.api.policies_api import PoliciesApi
from instana_client.api.releases_api import ReleasesApi
from instana_client.api.roles_api import RolesApi
from instana_client.api.sli_report_api import SLIReportApi
from instana_client.api.sli_settings_api import SLISettingsApi
from instana_client.api.slo_correction_configurations_api import SLOCorrectionConfigurationsApi
from instana_client.api.slo_correction_windows_api import SLOCorrectionWindowsApi
from instana_client.api.service_levels_alert_configuration_api import ServiceLevelsAlertConfigurationApi
from instana_client.api.service_levels_objective_slo_configurations_api import ServiceLevelsObjectiveSLOConfigurationsApi
from instana_client.api.service_levels_objective_slo_report_api import ServiceLevelsObjectiveSLOReportApi
from instana_client.api.session_settings_api import SessionSettingsApi
from instana_client.api.synthetic_alert_configuration_api import SyntheticAlertConfigurationApi
from instana_client.api.synthetic_calls_api import SyntheticCallsApi
from instana_client.api.synthetic_catalog_api import SyntheticCatalogApi
from instana_client.api.synthetic_metrics_api import SyntheticMetricsApi
from instana_client.api.synthetic_settings_api import SyntheticSettingsApi
from instana_client.api.synthetic_test_playback_results_api import SyntheticTestPlaybackResultsApi
from instana_client.api.teams_api import TeamsApi
from instana_client.api.usage_api import UsageApi
from instana_client.api.user_api import UserApi
from instana_client.api.website_analyze_api import WebsiteAnalyzeApi
from instana_client.api.website_catalog_api import WebsiteCatalogApi
from instana_client.api.website_configuration_api import WebsiteConfigurationApi
from instana_client.api.website_metrics_api import WebsiteMetricsApi




# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.service_levelse_alert_config_with_metadata import ServiceLevelseAlertConfigWithMetadata

class TestServiceLevelseAlertConfigWithMetadata(unittest.TestCase):
    """ServiceLevelseAlertConfigWithMetadata unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ServiceLevelseAlertConfigWithMetadata:
        """Test ServiceLevelseAlertConfigWithMetadata
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ServiceLevelseAlertConfigWithMetadata`
        """
        model = ServiceLevelseAlertConfigWithMetadata()
        if include_optional:
            return ServiceLevelseAlertConfigWithMetadata(
                alert_channel_ids = [
                    ''
                    ],
                burn_rate_config = [
                    instana_client.models.service_levels_burn_rate_config.ServiceLevelsBurnRateConfig(
                        alert_window_type = 'SINGLE', 
                        duration = 56, 
                        duration_unit_type = 'millisecond', 
                        threshold = instana_client.models.service_levels_static_threshold_config.ServiceLevelsStaticThresholdConfig(
                            last_updated = 0, 
                            operator = '>', 
                            value = 0, ), )
                    ],
                burn_rate_time_windows = instana_client.models.service_levels_burn_rate_time_windows.ServiceLevelsBurnRateTimeWindows(
                    long_time_window = instana_client.models.alerting_time_window.AlertingTimeWindow(
                        duration = 56, 
                        duration_type = 'millisecond', 
                        duration_unit = 'millisecond', ), 
                    short_time_window = instana_client.models.alerting_time_window.AlertingTimeWindow(
                        duration = 56, 
                        duration_type = 'millisecond', 
                        duration_unit = 'millisecond', ), ),
                created = 1,
                custom_payload_fields = [
                    instana_client.models.custom_payload_field.CustomPayloadField(
                        key = '', 
                        type = '', )
                    ],
                description = '',
                enabled = True,
                id = '',
                initial_created = 1,
                name = '',
                read_only = True,
                rule = instana_client.models.service_levels_alert_rule.ServiceLevelsAlertRule(
                    alert_type = '', ),
                severity = 56,
                slo_ids = [
                    ''
                    ],
                threshold = instana_client.models.static_threshold.StaticThreshold(),
                time_threshold = instana_client.models.service_levels_time_threshold.ServiceLevelsTimeThreshold(
                    expiry = 56, 
                    time_window = 56, ),
                triggering = True
            )
        else:
            return ServiceLevelseAlertConfigWithMetadata(
                alert_channel_ids = [
                    ''
                    ],
                custom_payload_fields = [
                    instana_client.models.custom_payload_field.CustomPayloadField(
                        key = '', 
                        type = '', )
                    ],
                description = '',
                id = '',
                name = '',
                rule = instana_client.models.service_levels_alert_rule.ServiceLevelsAlertRule(
                    alert_type = '', ),
                severity = 56,
                slo_ids = [
                    ''
                    ],
                time_threshold = instana_client.models.service_levels_time_threshold.ServiceLevelsTimeThreshold(
                    expiry = 56, 
                    time_window = 56, ),
        )
        """

    def testServiceLevelseAlertConfigWithMetadata(self):
        """Test ServiceLevelseAlertConfigWithMetadata"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.business_activity import BusinessActivity

class TestBusinessActivity(unittest.TestCase):
    """BusinessActivity unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> BusinessActivity:
        """Test BusinessActivity
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `BusinessActivity`
        """
        model = BusinessActivity()
        if include_optional:
            return BusinessActivity(
                activity_id = 'ApproveInvoiceTask',
                activity_name = 'Approve Invoice',
                activity_start = 1680559706000,
                activity_type = 'userTask',
                call_id = '',
                endpoint_ids = [
                    ''
                    ],
                process_definition_id = 'invoice:2:aa2bbbcc-bb04-11ee-9d4e-0242ac110002',
                process_definition_name = 'Invoice Approval',
                root_process_instance_id = '486cbb3b-e633-11ee-8707-0242ac110002'
            )
        else:
            return BusinessActivity(
                activity_type = 'userTask',
        )
        """

    def testBusinessActivity(self):
        """Test BusinessActivity"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.metrics_result import MetricsResult

class TestMetricsResult(unittest.TestCase):
    """MetricsResult unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> MetricsResult:
        """Test MetricsResult
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `MetricsResult`
        """
        model = MetricsResult()
        if include_optional:
            return MetricsResult(
                metrics_result = [
                    instana_client.models.metrics_result_item.MetricsResultItem(
                        custom_tags = {
                            'key' : ''
                            }, 
                        metrics = [
                            {
                                'key' : None
                                }
                            ], 
                        run_type = '', 
                        tests = [
                            instana_client.models.metrics_test_result_item.MetricsTestResultItem(
                                application_id = '', 
                                application_ids = [
                                    ''
                                    ], 
                                location_id = [
                                    ''
                                    ], 
                                mobile_application_ids = [
                                    ''
                                    ], 
                                service_id = '', 
                                test_id = '', 
                                test_name = '', 
                                website_ids = [
                                    ''
                                    ], )
                            ], )
                    ]
            )
        else:
            return MetricsResult(
                metrics_result = [
                    instana_client.models.metrics_result_item.MetricsResultItem(
                        custom_tags = {
                            'key' : ''
                            }, 
                        metrics = [
                            {
                                'key' : None
                                }
                            ], 
                        run_type = '', 
                        tests = [
                            instana_client.models.metrics_test_result_item.MetricsTestResultItem(
                                application_id = '', 
                                application_ids = [
                                    ''
                                    ], 
                                location_id = [
                                    ''
                                    ], 
                                mobile_application_ids = [
                                    ''
                                    ], 
                                service_id = '', 
                                test_id = '', 
                                test_name = '', 
                                website_ids = [
                                    ''
                                    ], )
                            ], )
                    ],
        )
        """

    def testMetricsResult(self):
        """Test MetricsResult"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class ValidatedAlertingChannelInputInfo(BaseModel):
    """
    ValidatedAlertingChannelInputInfo
    """ # noqa: E501
    enabled: Optional[StrictBool] = Field(default=None, description="Flag to indicate whether or not the configuration is enabled.")
    entity_id: Optional[StrictStr] = Field(default=None, description="The entity ID in case of Smart Alerts, such as for the application, website or mobile app.", alias="entityId")
    event_types: Optional[Annotated[List[StrictStr], Field(min_length=0, max_length=6)]] = Field(default=None, description="The selected event types, if applicable.", alias="eventTypes")
    id: Annotated[str, Field(min_length=0, strict=True, max_length=64)] = Field(description="ID of the alert configuration.")
    invalid: Optional[StrictBool] = Field(default=None, description="Flag to indicate whether the specified query is invalid.")
    label: Annotated[str, Field(min_length=0, strict=True, max_length=256)] = Field(description="The name of the alert configuration.")
    query: Optional[Annotated[str, Field(min_length=0, strict=True, max_length=2048)]] = Field(default=None, description="The DFQ used in the alert configuration, if applicable.")
    selected_events: Optional[StrictInt] = Field(default=None, description="The number of selected events, if applicable.", alias="selectedEvents")
    type: StrictStr = Field(description="The alert type.")
    __properties: ClassVar[List[str]] = ["enabled", "entityId", "eventTypes", "id", "invalid", "label", "query", "selectedEvents", "type"]

    @field_validator('event_types')
    def event_types_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['incident', 'critical', 'warning', 'change', 'online', 'offline', 'agent_monitoring_issue', 'cve_issue', 'none']):
                raise ValueError("each list item must be one of ('incident', 'critical', 'warning', 'change', 'online', 'offline', 'agent_monitoring_issue', 'cve_issue', 'none')")
        return value

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['Alert', 'WebsiteSmartAlert', 'MobileSmartAlert', 'ApplicationSmartAlert', 'GlobalApplicationSmartAlert', 'SyntheticSmartAlert', 'InfraSmartAlert', 'ServiceLevelSmartAlert', 'LogSmartAlert']):
            raise ValueError("must be one of enum values ('Alert', 'WebsiteSmartAlert', 'MobileSmartAlert', 'ApplicationSmartAlert', 'GlobalApplicationSmartAlert', 'SyntheticSmartAlert', 'InfraSmartAlert', 'ServiceLevelSmartAlert', 'LogSmartAlert')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ValidatedAlertingChannelInputInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ValidatedAlertingChannelInputInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "enabled": obj.get("enabled"),
            "entityId": obj.get("entityId"),
            "eventTypes": obj.get("eventTypes"),
            "id": obj.get("id"),
            "invalid": obj.get("invalid"),
            "label": obj.get("label"),
            "query": obj.get("query"),
            "selectedEvents": obj.get("selectedEvents"),
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.service_now_enhanced_integration import ServiceNowEnhancedIntegration

class TestServiceNowEnhancedIntegration(unittest.TestCase):
    """ServiceNowEnhancedIntegration unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ServiceNowEnhancedIntegration:
        """Test ServiceNowEnhancedIntegration
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ServiceNowEnhancedIntegration`
        """
        model = ServiceNowEnhancedIntegration()
        if include_optional:
            return ServiceNowEnhancedIntegration(
                auto_close_incidents = True,
                enable_send_instana_notes = True,
                enable_send_service_now_activities = True,
                enable_send_service_now_work_notes = True,
                instana_url = '',
                manually_closed_incidents = True,
                password = '',
                resolution_of_incident = True,
                service_now_url = '',
                snow_status_on_close_event = 56,
                tenant = '',
                unit = '',
                username = ''
            )
        else:
            return ServiceNowEnhancedIntegration(
                password = '',
                service_now_url = '',
                tenant = '',
                unit = '',
                username = '',
        )
        """

    def testServiceNowEnhancedIntegration(self):
        """Test ServiceNowEnhancedIntegration"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.api_restricted_application_filter import ApiRestrictedApplicationFilter
from instana_client.models.scope_binding import ScopeBinding
from typing import Optional, Set
from typing_extensions import Self

class ApiPermissionSet(BaseModel):
    """
    ApiPermissionSet
    """ # noqa: E501
    action_filter: Optional[ScopeBinding] = Field(default=None, alias="actionFilter")
    application_ids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="applicationIds")
    business_perspective_ids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="businessPerspectiveIds")
    infra_dfq_filter: ScopeBinding = Field(alias="infraDfqFilter")
    kubernetes_cluster_uuids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="kubernetesClusterUUIDs")
    kubernetes_namespace_uids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="kubernetesNamespaceUIDs")
    mobile_app_ids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="mobileAppIds")
    permissions: Annotated[List[StrictStr], Field(min_length=0, max_length=1024)]
    restricted_application_filter: Optional[ApiRestrictedApplicationFilter] = Field(default=None, alias="restrictedApplicationFilter")
    slo_ids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="sloIds")
    synthetic_credential_keys: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="syntheticCredentialKeys")
    synthetic_test_ids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="syntheticTestIds")
    website_ids: Annotated[List[ScopeBinding], Field(min_length=0, max_length=1024)] = Field(alias="websiteIds")
    __properties: ClassVar[List[str]] = ["actionFilter", "applicationIds", "businessPerspectiveIds", "infraDfqFilter", "kubernetesClusterUUIDs", "kubernetesNamespaceUIDs", "mobileAppIds", "permissions", "restrictedApplicationFilter", "sloIds", "syntheticCredentialKeys", "syntheticTestIds", "websiteIds"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ApiPermissionSet from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of action_filter
        if self.action_filter:
            _dict['actionFilter'] = self.action_filter.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in application_ids (list)
        _items = []
        if self.application_ids:
            for _item_application_ids in self.application_ids:
                if _item_application_ids:
                    _items.append(_item_application_ids.to_dict())
            _dict['applicationIds'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in business_perspective_ids (list)
        _items = []
        if self.business_perspective_ids:
            for _item_business_perspective_ids in self.business_perspective_ids:
                if _item_business_perspective_ids:
                    _items.append(_item_business_perspective_ids.to_dict())
            _dict['businessPerspectiveIds'] = _items
        # override the default output from pydantic by calling `to_dict()` of infra_dfq_filter
        if self.infra_dfq_filter:
            _dict['infraDfqFilter'] = self.infra_dfq_filter.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in kubernetes_cluster_uuids (list)
        _items = []
        if self.kubernetes_cluster_uuids:
            for _item_kubernetes_cluster_uuids in self.kubernetes_cluster_uuids:
                if _item_kubernetes_cluster_uuids:
                    _items.append(_item_kubernetes_cluster_uuids.to_dict())
            _dict['kubernetesClusterUUIDs'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in kubernetes_namespace_uids (list)
        _items = []
        if self.kubernetes_namespace_uids:
            for _item_kubernetes_namespace_uids in self.kubernetes_namespace_uids:
                if _item_kubernetes_namespace_uids:
                    _items.append(_item_kubernetes_namespace_uids.to_dict())
            _dict['kubernetesNamespaceUIDs'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in mobile_app_ids (list)
        _items = []
        if self.mobile_app_ids:
            for _item_mobile_app_ids in self.mobile_app_ids:
                if _item_mobile_app_ids:
                    _items.append(_item_mobile_app_ids.to_dict())
            _dict['mobileAppIds'] = _items
        # override the default output from pydantic by calling `to_dict()` of restricted_application_filter
        if self.restricted_application_filter:
            _dict['restrictedApplicationFilter'] = self.restricted_application_filter.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in slo_ids (list)
        _items = []
        if self.slo_ids:
            for _item_slo_ids in self.slo_ids:
                if _item_slo_ids:
                    _items.append(_item_slo_ids.to_dict())
            _dict['sloIds'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in synthetic_credential_keys (list)
        _items = []
        if self.synthetic_credential_keys:
            for _item_synthetic_credential_keys in self.synthetic_credential_keys:
                if _item_synthetic_credential_keys:
                    _items.append(_item_synthetic_credential_keys.to_dict())
            _dict['syntheticCredentialKeys'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in synthetic_test_ids (list)
        _items = []
        if self.synthetic_test_ids:
            for _item_synthetic_test_ids in self.synthetic_test_ids:
                if _item_synthetic_test_ids:
                    _items.append(_item_synthetic_test_ids.to_dict())
            _dict['syntheticTestIds'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in website_ids (list)
        _items = []
        if self.website_ids:
            for _item_website_ids in self.website_ids:
                if _item_website_ids:
                    _items.append(_item_website_ids.to_dict())
            _dict['websiteIds'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ApiPermissionSet from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "actionFilter": ScopeBinding.from_dict(obj["actionFilter"]) if obj.get("actionFilter") is not None else None,
            "applicationIds": [ScopeBinding.from_dict(_item) for _item in obj["applicationIds"]] if obj.get("applicationIds") is not None else None,
            "businessPerspectiveIds": [ScopeBinding.from_dict(_item) for _item in obj["businessPerspectiveIds"]] if obj.get("businessPerspectiveIds") is not None else None,
            "infraDfqFilter": ScopeBinding.from_dict(obj["infraDfqFilter"]) if obj.get("infraDfqFilter") is not None else None,
            "kubernetesClusterUUIDs": [ScopeBinding.from_dict(_item) for _item in obj["kubernetesClusterUUIDs"]] if obj.get("kubernetesClusterUUIDs") is not None else None,
            "kubernetesNamespaceUIDs": [ScopeBinding.from_dict(_item) for _item in obj["kubernetesNamespaceUIDs"]] if obj.get("kubernetesNamespaceUIDs") is not None else None,
            "mobileAppIds": [ScopeBinding.from_dict(_item) for _item in obj["mobileAppIds"]] if obj.get("mobileAppIds") is not None else None,
            "permissions": obj.get("permissions"),
            "restrictedApplicationFilter": ApiRestrictedApplicationFilter.from_dict(obj["restrictedApplicationFilter"]) if obj.get("restrictedApplicationFilter") is not None else None,
            "sloIds": [ScopeBinding.from_dict(_item) for _item in obj["sloIds"]] if obj.get("sloIds") is not None else None,
            "syntheticCredentialKeys": [ScopeBinding.from_dict(_item) for _item in obj["syntheticCredentialKeys"]] if obj.get("syntheticCredentialKeys") is not None else None,
            "syntheticTestIds": [ScopeBinding.from_dict(_item) for _item in obj["syntheticTestIds"]] if obj.get("syntheticTestIds") is not None else None,
            "websiteIds": [ScopeBinding.from_dict(_item) for _item in obj["websiteIds"]] if obj.get("websiteIds") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from instana_client.models.mobile_app_time_threshold import MobileAppTimeThreshold
from typing import Optional, Set
from typing_extensions import Self

class UserImpactMobileAppTimeThreshold(MobileAppTimeThreshold):
    """
    UserImpactMobileAppTimeThreshold
    """ # noqa: E501
    impact_measurement_method: StrictStr = Field(alias="impactMeasurementMethod")
    user_percentage: Optional[Union[Annotated[float, Field(le=1.0, strict=True, gt=0.0)], Annotated[int, Field(le=1, strict=True, gt=0)]]] = Field(default=None, alias="userPercentage")
    users: Optional[Annotated[int, Field(strict=True, ge=1)]] = None
    __properties: ClassVar[List[str]] = ["timeWindow", "type", "impactMeasurementMethod", "userPercentage", "users"]

    @field_validator('impact_measurement_method')
    def impact_measurement_method_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['AGGREGATED', 'PER_WINDOW']):
            raise ValueError("must be one of enum values ('AGGREGATED', 'PER_WINDOW')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UserImpactMobileAppTimeThreshold from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UserImpactMobileAppTimeThreshold from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "timeWindow": obj.get("timeWindow"),
            "type": obj.get("type"),
            "impactMeasurementMethod": obj.get("impactMeasurementMethod"),
            "userPercentage": obj.get("userPercentage"),
            "users": obj.get("users")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.website_monitoring_beacon import WebsiteMonitoringBeacon

class TestWebsiteMonitoringBeacon(unittest.TestCase):
    """WebsiteMonitoringBeacon unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> WebsiteMonitoringBeacon:
        """Test WebsiteMonitoringBeacon
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `WebsiteMonitoringBeacon`
        """
        model = WebsiteMonitoringBeacon()
        if include_optional:
            return WebsiteMonitoringBeacon(
                accuracy_radius = -1,
                accurate_timings_available = True,
                agent_version = '',
                app_cache_time = -1,
                backend_time = -1,
                backend_trace_id = '',
                batch_size = 1,
                beacon_id = '',
                browser_name = '',
                browser_version = '',
                bytes_ingested = 56,
                cache_interaction = '',
                children_time = -1,
                city = '',
                clock_skew = -1,
                component_stack = '',
                connection_type = '',
                continent = '',
                continent_code = '',
                country = '',
                country_code = '',
                csp_blocked_uri = '',
                csp_column_number = 56,
                csp_disposition = '',
                csp_effective_directive = '',
                csp_line_number = 56,
                csp_original_policy = '',
                csp_sample = '',
                csp_source_file = '',
                cumulative_layout_shift = 1.337,
                custom_event_name = '',
                custom_metric = 1.337,
                decoded_body_size = -1,
                deprecations = [
                    ''
                    ],
                device_type = '',
                dns_time = -1,
                dom_time = -1,
                duration = 0,
                encoded_body_size = -1,
                error_count = 0,
                error_id = '',
                error_message = '',
                error_type = '',
                first_contentful_paint_time = -1,
                first_input_delay_time = -1,
                first_paint_time = -1,
                frontend_time = -1,
                graphql_operation_name = '',
                graphql_operation_type = '',
                http_call_asynchronous = True,
                http_call_correlation_attempted = True,
                http_call_headers = {
                    'key' : ''
                    },
                http_call_method = '',
                http_call_origin = '',
                http_call_path = '',
                http_call_status = -1,
                http_call_url = '',
                initiator = '',
                interaction_next_paint = 56,
                internal_meta = {
                    'key' : ''
                    },
                label = '',
                largest_contentful_paint_time = -1,
                latitude = 1.337,
                location_origin = '',
                location_path = '',
                location_url = '',
                longitude = 1.337,
                meta = {
                    'key' : ''
                    },
                on_load_time = -1,
                os_name = '',
                os_version = '',
                page = '',
                page_load_id = '',
                parent_beacon_id = '',
                parsed_stack_trace = [
                    instana_client.models.js_stack_trace_line.JsStackTraceLine(
                        column = -1, 
                        file = '', 
                        line = -1, 
                        name = '', 
                        translation_explanation = '', 
                        translation_status = -1, )
                    ],
                phase = '',
                processing_time = -1,
                redirect_time = -1,
                request_time = -1,
                resource_type = '',
                response_time = -1,
                session_id = '',
                snippet_version = '',
                ssl_time = -1,
                stack_trace = '',
                stack_trace_parsing_status = -1,
                stack_trace_readability = 0,
                subdivision = '',
                subdivision_code = '',
                tcp_time = -1,
                timestamp = 1,
                transfer_size = -1,
                type = '',
                unload_time = -1,
                use_features = [
                    ''
                    ],
                user_email = '',
                user_id = '',
                user_ip = '',
                user_languages = [
                    ''
                    ],
                user_name = '',
                website_id = '',
                website_label = '',
                window_height = -1,
                window_hidden = True,
                window_width = -1
            )
        else:
            return WebsiteMonitoringBeacon(
                beacon_id = '',
                location_origin = '',
                location_url = '',
                page_load_id = '',
                type = '',
                website_id = '',
                website_label = '',
        )
        """

    def testWebsiteMonitoringBeacon(self):
        """Test WebsiteMonitoringBeacon"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.span_relation import SpanRelation

class TestSpanRelation(unittest.TestCase):
    """SpanRelation unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SpanRelation:
        """Test SpanRelation
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SpanRelation`
        """
        model = SpanRelation()
        if include_optional:
            return SpanRelation(
                applications = [
                    instana_client.models.application.Application(
                        boundary_scope = '', 
                        entity_type = 'APPLICATION', 
                        id = '', 
                        label = '', )
                    ],
                endpoint = instana_client.models.endpoint.Endpoint(
                    entity_type = 'APPLICATION', 
                    id = '', 
                    is_synthetic = True, 
                    label = '', 
                    service_id = '', 
                    synthetic = True, 
                    synthetic_type = 'NON_SYNTHETIC', 
                    technologies = [
                        ''
                        ], 
                    type = 'UNDEFINED', ),
                physical_context = instana_client.models.physical_context.PhysicalContext(
                    cloudfoundry = instana_client.models.cloudfoundry_physical_context.CloudfoundryPhysicalContext(
                        application = instana_client.models.snapshot_preview.SnapshotPreview(
                            data = {
                                'key' : None
                                }, 
                            id = '', 
                            label = '', 
                            plugin = '', 
                            time = 1, ), 
                        cf_instance_index = '', 
                        organization = instana_client.models.snapshot_preview.SnapshotPreview(
                            id = '', 
                            label = '', 
                            plugin = '', 
                            time = 1, ), 
                        space = , ), 
                    cluster = , 
                    container = , 
                    host = , 
                    kubernetes = instana_client.models.kubernetes_physical_context.KubernetesPhysicalContext(
                        namespace = , 
                        node = , 
                        pod = , ), 
                    process = , ),
                service = instana_client.models.service.Service(
                    entity_type = 'APPLICATION', 
                    id = '', 
                    label = '', 
                    snapshot_ids = [
                        ''
                        ], 
                    technologies = [
                        ''
                        ], 
                    types = [
                        'UNDEFINED'
                        ], )
            )
        else:
            return SpanRelation(
                applications = [
                    instana_client.models.application.Application(
                        boundary_scope = '', 
                        entity_type = 'APPLICATION', 
                        id = '', 
                        label = '', )
                    ],
        )
        """

    def testSpanRelation(self):
        """Test SpanRelation"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.policy_runnable import PolicyRunnable

class TestPolicyRunnable(unittest.TestCase):
    """PolicyRunnable unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PolicyRunnable:
        """Test PolicyRunnable
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PolicyRunnable`
        """
        model = PolicyRunnable()
        if include_optional:
            return PolicyRunnable(
                id = '',
                run_configuration = instana_client.models.run_configuration.RunConfiguration(
                    actions = [
                        instana_client.models.action_configuration.ActionConfiguration(
                            action = instana_client.models.action.Action(
                                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                description = '', 
                                fields = [
                                    instana_client.models.field.Field(
                                        description = '', 
                                        encoding = '', 
                                        name = '', 
                                        secured = True, 
                                        value = '', )
                                    ], 
                                id = '0', 
                                input_parameters = [
                                    instana_client.models.parameter.Parameter(
                                        description = '', 
                                        hidden = True, 
                                        label = '', 
                                        name = '', 
                                        required = True, 
                                        secured = True, 
                                        type = '', 
                                        value = '', )
                                    ], 
                                metadata = instana_client.models.meta_data.MetaData(
                                    ai = [
                                        {
                                            'key' : None
                                            }
                                        ], 
                                    ai_originated = True, 
                                    built_in = True, 
                                    read_only = True, 
                                    sensor_imported = True, ), 
                                modified_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                name = '0', 
                                tags = [
                                    ''
                                    ], 
                                type = 'SCRIPT', ), 
                            agent_id = '', 
                            input_parameter_values = [
                                instana_client.models.parameter_value.ParameterValue(
                                    name = '', 
                                    value = '', )
                                ], )
                        ], ),
                type = 'action'
            )
        else:
            return PolicyRunnable(
                id = '',
                run_configuration = instana_client.models.run_configuration.RunConfiguration(
                    actions = [
                        instana_client.models.action_configuration.ActionConfiguration(
                            action = instana_client.models.action.Action(
                                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                description = '', 
                                fields = [
                                    instana_client.models.field.Field(
                                        description = '', 
                                        encoding = '', 
                                        name = '', 
                                        secured = True, 
                                        value = '', )
                                    ], 
                                id = '0', 
                                input_parameters = [
                                    instana_client.models.parameter.Parameter(
                                        description = '', 
                                        hidden = True, 
                                        label = '', 
                                        name = '', 
                                        required = True, 
                                        secured = True, 
                                        type = '', 
                                        value = '', )
                                    ], 
                                metadata = instana_client.models.meta_data.MetaData(
                                    ai = [
                                        {
                                            'key' : None
                                            }
                                        ], 
                                    ai_originated = True, 
                                    built_in = True, 
                                    read_only = True, 
                                    sensor_imported = True, ), 
                                modified_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                                name = '0', 
                                tags = [
                                    ''
                                    ], 
                                type = 'SCRIPT', ), 
                            agent_id = '', 
                            input_parameter_values = [
                                instana_client.models.parameter_value.ParameterValue(
                                    name = '', 
                                    value = '', )
                                ], )
                        ], ),
                type = 'action',
        )
        """

    def testPolicyRunnable(self):
        """Test PolicyRunnable"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from instana_client.models.cursor_pagination_infra_explore_cursor import CursorPaginationInfraExploreCursor
from instana_client.models.infra_metric_configuration import InfraMetricConfiguration
from instana_client.models.order import Order
from instana_client.models.tag_filter_expression_element import TagFilterExpressionElement
from instana_client.models.time_frame import TimeFrame
from typing import Optional, Set
from typing_extensions import Self

class GetInfrastructureGroupsQuery(BaseModel):
    """
    GetInfrastructureGroupsQuery
    """ # noqa: E501
    group_by: Annotated[List[StrictStr], Field(min_length=0, max_length=5)] = Field(description="Entity grouping tag. Use the Instana Analyze Infrastructure dashboard to determine the entity grouping tag name.", alias="groupBy")
    metrics: Optional[Annotated[List[InfraMetricConfiguration], Field(min_length=0, max_length=10)]] = None
    missing_placeholder: Optional[StrictStr] = Field(default=None, alias="missingPlaceholder")
    order: Optional[Order] = None
    pagination: CursorPaginationInfraExploreCursor
    tag_filter_expression: TagFilterExpressionElement = Field(alias="tagFilterExpression")
    time_frame: TimeFrame = Field(alias="timeFrame")
    type: Optional[StrictStr] = Field(default=None, description="Type of entities")
    __properties: ClassVar[List[str]] = ["groupBy", "metrics", "missingPlaceholder", "order", "pagination", "tagFilterExpression", "timeFrame", "type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetInfrastructureGroupsQuery from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in metrics (list)
        _items = []
        if self.metrics:
            for _item_metrics in self.metrics:
                if _item_metrics:
                    _items.append(_item_metrics.to_dict())
            _dict['metrics'] = _items
        # override the default output from pydantic by calling `to_dict()` of order
        if self.order:
            _dict['order'] = self.order.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pagination
        if self.pagination:
            _dict['pagination'] = self.pagination.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tag_filter_expression
        if self.tag_filter_expression:
            _dict['tagFilterExpression'] = self.tag_filter_expression.to_dict()
        # override the default output from pydantic by calling `to_dict()` of time_frame
        if self.time_frame:
            _dict['timeFrame'] = self.time_frame.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetInfrastructureGroupsQuery from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "groupBy": obj.get("groupBy"),
            "metrics": [InfraMetricConfiguration.from_dict(_item) for _item in obj["metrics"]] if obj.get("metrics") is not None else None,
            "missingPlaceholder": obj.get("missingPlaceholder"),
            "order": Order.from_dict(obj["order"]) if obj.get("order") is not None else None,
            "pagination": CursorPaginationInfraExploreCursor.from_dict(obj["pagination"]) if obj.get("pagination") is not None else None,
            "tagFilterExpression": TagFilterExpressionElement.from_dict(obj["tagFilterExpression"]) if obj.get("tagFilterExpression") is not None else None,
            "timeFrame": TimeFrame.from_dict(obj["timeFrame"]) if obj.get("timeFrame") is not None else None,
            "type": obj.get("type")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.access_log_response import AccessLogResponse

class TestAccessLogResponse(unittest.TestCase):
    """AccessLogResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> AccessLogResponse:
        """Test AccessLogResponse
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `AccessLogResponse`
        """
        model = AccessLogResponse()
        if include_optional:
            return AccessLogResponse(
                entries = [
                    instana_client.models.access_log_entry.AccessLogEntry(
                        action = 'GRANT_TEMP_ACCESS', 
                        email = '', 
                        full_name = '', 
                        message = '', 
                        meta = {
                            'key' : None
                            }, 
                        tenant_id = '', 
                        tenant_unit_id = '', 
                        timestamp = 1118583367265, )
                    ],
                total = 56
            )
        else:
            return AccessLogResponse(
        )
        """

    def testAccessLogResponse(self):
        """Test AccessLogResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from instana_client.models.dns_filter_query_time import DNSFilterQueryTime
from instana_client.models.synthetic_resource_update_list_dns_filter_target_value_list_dns_filter_target_value import SyntheticResourceUpdateListDNSFilterTargetValueListDNSFilterTargetValue
from instana_client.models.synthetic_type_configuration_update import SyntheticTypeConfigurationUpdate
from typing import Optional, Set
from typing_extensions import Self

class DNSConfigurationUpdate(SyntheticTypeConfigurationUpdate):
    """
    DNSConfigurationUpdate
    """ # noqa: E501
    accept_cname: Optional[StrictBool] = Field(default=None, alias="acceptCNAME")
    lookup: Optional[StrictStr] = None
    lookup_server_name: Optional[StrictBool] = Field(default=None, alias="lookupServerName")
    port: Optional[StrictInt] = None
    query_time: Optional[DNSFilterQueryTime] = Field(default=None, alias="queryTime")
    query_type: Optional[StrictStr] = Field(default=None, alias="queryType")
    recursive_lookups: Optional[StrictBool] = Field(default=None, alias="recursiveLookups")
    server: Optional[StrictStr] = None
    server_retries: Optional[StrictInt] = Field(default=None, alias="serverRetries")
    target_values: Optional[SyntheticResourceUpdateListDNSFilterTargetValueListDNSFilterTargetValue] = Field(default=None, alias="targetValues")
    transport: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["markSyntheticCall", "retries", "retryInterval", "syntheticTest", "syntheticType", "timeout", "acceptCNAME", "lookup", "lookupServerName", "port", "queryTime", "queryType", "recursiveLookups", "server", "serverRetries", "targetValues", "transport"]

    @field_validator('query_type')
    def query_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['ALL', 'ALL_CONDITIONS', 'ANY', 'A', 'AAAA', 'CNAME', 'NS']):
            raise ValueError("must be one of enum values ('ALL', 'ALL_CONDITIONS', 'ANY', 'A', 'AAAA', 'CNAME', 'NS')")
        return value

    @field_validator('transport')
    def transport_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['TCP', 'UDP']):
            raise ValueError("must be one of enum values ('TCP', 'UDP')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DNSConfigurationUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of query_time
        if self.query_time:
            _dict['queryTime'] = self.query_time.to_dict()
        # override the default output from pydantic by calling `to_dict()` of target_values
        if self.target_values:
            _dict['targetValues'] = self.target_values.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DNSConfigurationUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "markSyntheticCall": obj.get("markSyntheticCall"),
            "retries": obj.get("retries"),
            "retryInterval": obj.get("retryInterval"),
            "syntheticTest": obj.get("syntheticTest"),
            "syntheticType": obj.get("syntheticType"),
            "timeout": obj.get("timeout"),
            "acceptCNAME": obj.get("acceptCNAME"),
            "lookup": obj.get("lookup"),
            "lookupServerName": obj.get("lookupServerName"),
            "port": obj.get("port"),
            "queryTime": DNSFilterQueryTime.from_dict(obj["queryTime"]) if obj.get("queryTime") is not None else None,
            "queryType": obj.get("queryType"),
            "recursiveLookups": obj.get("recursiveLookups"),
            "server": obj.get("server"),
            "serverRetries": obj.get("serverRetries"),
            "targetValues": SyntheticResourceUpdateListDNSFilterTargetValueListDNSFilterTargetValue.from_dict(obj["targetValues"]) if obj.get("targetValues") is not None else None,
            "transport": obj.get("transport")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.get_infrastructure_groups_query import GetInfrastructureGroupsQuery

class TestGetInfrastructureGroupsQuery(unittest.TestCase):
    """GetInfrastructureGroupsQuery unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> GetInfrastructureGroupsQuery:
        """Test GetInfrastructureGroupsQuery
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `GetInfrastructureGroupsQuery`
        """
        model = GetInfrastructureGroupsQuery()
        if include_optional:
            return GetInfrastructureGroupsQuery(
                group_by = [
                    ''
                    ],
                metrics = [
                    instana_client.models.infra_metric_configuration.InfraMetricConfiguration()
                    ],
                missing_placeholder = '',
                order = instana_client.models.order.Order(
                    by = '', 
                    collation = '', 
                    direction = 'ASC', ),
                pagination = instana_client.models.cursor_pagination_infra_explore_cursor.CursorPaginationInfraExploreCursor(
                    cursor = instana_client.models.cursor.cursor(), 
                    retrieval_size = 56, ),
                tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                    type = '', ),
                time_frame = instana_client.models.time_frame.TimeFrame(
                    to = 56, 
                    window_size = 0, ),
                type = ''
            )
        else:
            return GetInfrastructureGroupsQuery(
                group_by = [
                    ''
                    ],
                pagination = instana_client.models.cursor_pagination_infra_explore_cursor.CursorPaginationInfraExploreCursor(
                    cursor = instana_client.models.cursor.cursor(), 
                    retrieval_size = 56, ),
                tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                    type = '', ),
                time_frame = instana_client.models.time_frame.TimeFrame(
                    to = 56, 
                    window_size = 0, ),
        )
        """

    def testGetInfrastructureGroupsQuery(self):
        """Test GetInfrastructureGroupsQuery"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.synthetic_alert_config_with_metadata import SyntheticAlertConfigWithMetadata

class TestSyntheticAlertConfigWithMetadata(unittest.TestCase):
    """SyntheticAlertConfigWithMetadata unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SyntheticAlertConfigWithMetadata:
        """Test SyntheticAlertConfigWithMetadata
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SyntheticAlertConfigWithMetadata`
        """
        model = SyntheticAlertConfigWithMetadata()
        if include_optional:
            return SyntheticAlertConfigWithMetadata(
                alert_channel_ids = [
                    ''
                    ],
                created = 1,
                custom_payload_fields = [
                    instana_client.models.custom_payload_field.CustomPayloadField(
                        key = '', 
                        type = '', )
                    ],
                description = '',
                enabled = True,
                grace_period = 56,
                id = '',
                initial_created = 1,
                name = '',
                read_only = True,
                rule = instana_client.models.synthetic_alert_rule.SyntheticAlertRule(
                    aggregation = 'SUM', 
                    alert_type = '', 
                    metric_name = '', ),
                severity = 5,
                synthetic_test_ids = [
                    ''
                    ],
                tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                    type = '', ),
                time_threshold = instana_client.models.synthetic_time_threshold.SyntheticTimeThreshold(
                    type = '', 
                    violations_count = 1, )
            )
        else:
            return SyntheticAlertConfigWithMetadata(
                alert_channel_ids = [
                    ''
                    ],
                custom_payload_fields = [
                    instana_client.models.custom_payload_field.CustomPayloadField(
                        key = '', 
                        type = '', )
                    ],
                description = '',
                id = '',
                name = '',
                rule = instana_client.models.synthetic_alert_rule.SyntheticAlertRule(
                    aggregation = 'SUM', 
                    alert_type = '', 
                    metric_name = '', ),
                synthetic_test_ids = [
                    ''
                    ],
                tag_filter_expression = instana_client.models.tag_filter_expression_element.TagFilterExpressionElement(
                    type = '', ),
                time_threshold = instana_client.models.synthetic_time_threshold.SyntheticTimeThreshold(
                    type = '', 
                    violations_count = 1, ),
        )
        """

    def testSyntheticAlertConfigWithMetadata(self):
        """Test SyntheticAlertConfigWithMetadata"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class ServiceMapConnection(BaseModel):
    """
    A list which indicates which services are consumers and which are providers in the communication chain.
    """ # noqa: E501
    calls: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=None, description="The number of calls between the 2 services. Eg: The number of calls from `24558242fdeb52571cfb9fa42f1b334aa65d7e61` service to `3feb3dcd206c166ef2b41c707e0cd38d7cd325aa` service over the mentioned timeframe is `200`.   If no timeframe (`to` and `windowSize` in query paramter) is mentioned, the timeframe is taken as last hour. ")
    error_rate: Optional[Union[Annotated[float, Field(le=1, strict=True, ge=0)], Annotated[int, Field(le=1, strict=True, ge=0)]]] = Field(default=None, description="The error rate of the calls between the 2 services. Eg: The error rate of calls from `24558242fdeb52571cfb9fa42f1b334aa65d7e61` service to `3feb3dcd206c166ef2b41c707e0cd38d7cd325aa` service over the mentioned timeframe is `0.1`.   The value is between 0 and 1.   If no timeframe (`to` and `windowSize` in query paramter) is mentioned, the timeframe is taken as last hour. ", alias="errorRate")
    var_from: StrictStr = Field(description="The service that initiates a request to another service. It contains a unique service id. Eg: `24558242fdeb52571cfb9fa42f1b334aa65d7e61`. ", alias="from")
    latency: Optional[Union[Annotated[float, Field(strict=True, ge=0)], Annotated[int, Field(strict=True, ge=0)]]] = Field(default=None, description="The mean latency of the calls between the 2 services. Eg: The mean latency of calls from `24558242fdeb52571cfb9fa42f1b334aa65d7e61` service to `3feb3dcd206c166ef2b41c707e0cd38d7cd325aa` service over the mentioned timeframe is `4.46`.   The value is in milliseconds.   If no timeframe (`to` and `windowSize` in query paramter) is mentioned, the timeframe is taken as last hour. ")
    to: StrictStr = Field(description="The service that receives the request sent by the source service. It contains a unique service id. Eg: `3feb3dcd206c166ef2b41c707e0cd38d7cd325aa`. ")
    __properties: ClassVar[List[str]] = ["calls", "errorRate", "from", "latency", "to"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ServiceMapConnection from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ServiceMapConnection from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "calls": obj.get("calls"),
            "errorRate": obj.get("errorRate"),
            "from": obj.get("from"),
            "latency": obj.get("latency"),
            "to": obj.get("to")
        })
        return _obj





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.309.1268
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from importlib import import_module
from pydantic import BaseModel, ConfigDict, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Union
from typing import Optional, Set
from typing_extensions import Self

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from instana_client.models.fixed_http_path_segment_matching_rule import FixedHttpPathSegmentMatchingRule
    from instana_client.models.match_all_http_path_segment_matching_rule import MatchAllHttpPathSegmentMatchingRule
    from instana_client.models.path_parameter_http_path_segment_matching_rule import PathParameterHttpPathSegmentMatchingRule
    from instana_client.models.unsupported_http_path_segment_matching_rule import UnsupportedHttpPathSegmentMatchingRule

class HttpPathSegmentMatchingRule(BaseModel):
    """
    A list of path segment matching rules, each defining how a segment of the HTTP path should be matched. Each object in this array represents a segment rule, allowing for fixed segments, dynamic parameters, wildcards, or unsupported segments.   **UNSUPPORTED**: A path segment that is not recognized by the system.   **FIXED**: This type represents a static, unchanging part of the URL path. For example, `/api/{version}/users`, `api` and `users` would be `FIXED` segment.   **PARAMETER**: This type represents a variable part of the URL path, often used to capture specific parameters or IDs that change with each request. For example, `/api/{version}/users`, `version` would be `PARAMETER` segment. `version` can be `v1`, `v2`, `v3` etc.   **MATCH_ALL**: This type represents a wildcard, capturing all remaining segments from this point onward in the URL path. For example, `/api/{version}/users/*` — Matches all paths like `/api/v1/users/123`. `/api/v3/users/456` etc.   
    """ # noqa: E501
    type: StrictStr
    __properties: ClassVar[List[str]] = ["type"]

    @field_validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['UNSUPPORTED', 'FIXED', 'PARAMETER', 'MATCH_ALL']):
            raise ValueError("must be one of enum values ('UNSUPPORTED', 'FIXED', 'PARAMETER', 'MATCH_ALL')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    # JSON field name that stores the object type
    __discriminator_property_name: ClassVar[str] = 'type'

    # discriminator mappings
    __discriminator_value_class_map: ClassVar[Dict[str, str]] = {
        'FIXED': 'FixedHttpPathSegmentMatchingRule','MATCH_ALL': 'MatchAllHttpPathSegmentMatchingRule','PARAMETER': 'PathParameterHttpPathSegmentMatchingRule','UNSUPPORTED': 'UnsupportedHttpPathSegmentMatchingRule'
    }

    @classmethod
    def get_discriminator_value(cls, obj: Dict[str, Any]) -> Optional[str]:
        """Returns the discriminator value (object type) of the data"""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Union[FixedHttpPathSegmentMatchingRule, MatchAllHttpPathSegmentMatchingRule, PathParameterHttpPathSegmentMatchingRule, UnsupportedHttpPathSegmentMatchingRule]]:
        """Create an instance of HttpPathSegmentMatchingRule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Optional[Union[FixedHttpPathSegmentMatchingRule, MatchAllHttpPathSegmentMatchingRule, PathParameterHttpPathSegmentMatchingRule, UnsupportedHttpPathSegmentMatchingRule]]:
        """Create an instance of HttpPathSegmentMatchingRule from a dict"""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type ==  'FixedHttpPathSegmentMatchingRule':
            return import_module("instana_client.models.fixed_http_path_segment_matching_rule").FixedHttpPathSegmentMatchingRule.from_dict(obj)
        if object_type ==  'MatchAllHttpPathSegmentMatchingRule':
            return import_module("instana_client.models.match_all_http_path_segment_matching_rule").MatchAllHttpPathSegmentMatchingRule.from_dict(obj)
        if object_type ==  'PathParameterHttpPathSegmentMatchingRule':
            return import_module("instana_client.models.path_parameter_http_path_segment_matching_rule").PathParameterHttpPathSegmentMatchingRule.from_dict(obj)
        if object_type ==  'UnsupportedHttpPathSegmentMatchingRule':
            return import_module("instana_client.models.unsupported_http_path_segment_matching_rule").UnsupportedHttpPathSegmentMatchingRule.from_dict(obj)

        raise ValueError("HttpPathSegmentMatchingRule failed to lookup discriminator value from " +
                            json.dumps(obj) + ". Discriminator property name: " + cls.__discriminator_property_name +
                            ", mapping: " + json.dumps(cls.__discriminator_value_class_map))





# coding: utf-8

"""
    Instana REST API documentation

    Documentation for INSTANA REST API

    The version of the OpenAPI document: 1.306.1368
    Contact: support@instana.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from instana_client.models.alerting_configuration_with_last_updated import AlertingConfigurationWithLastUpdated

class TestAlertingConfigurationWithLastUpdated(unittest.TestCase):
    """AlertingConfigurationWithLastUpdated unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> AlertingConfigurationWithLastUpdated:
        """Test AlertingConfigurationWithLastUpdated
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `AlertingConfigurationWithLastUpdated`
        """
        model = AlertingConfigurationWithLastUpdated()
        if include_optional:
            return AlertingConfigurationWithLastUpdated(
                alert_name = '',
                custom_payload_fields = [
                    instana_client.models.static_string_field.StaticStringField()
                    ],
                event_filtering_configuration = instana_client.models.event_filtering_configuration.EventFilteringConfiguration(
                    application_alert_config_ids = [
                        ''
                        ], 
                    event_types = [
                        'incident'
                        ], 
                    query = '', 
                    rule_ids = [
                        ''
                        ], ),
                id = '',
                include_entity_name_in_legacy_alerts = True,
                integration_ids = [
                    ''
                    ],
                last_updated = 1,
                mute_until = 56
            )
        else:
            return AlertingConfigurationWithLastUpdated(
                alert_name = '',
                custom_payload_fields = [
                    instana_client.models.static_string_field.StaticStringField()
                    ],
                event_filtering_configuration = instana_client.models.event_filtering_configuration.EventFilteringConfiguration(
                    application_alert_config_ids = [
                        ''
                        ], 
                    event_types = [
                        'incident'
                        ], 
                    query = '', 
                    rule_ids = [
                        ''
                        ], ),
                id = '',
                integration_ids = [
                    ''
                    ],
        )
        """

    def testAlertingConfigurationWithLastUpdated(self):
        """Test AlertingConfigurationWithLastUpdated"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
